//---------------------------------------------------------------------------
//
//   SetLocalAxis
//
//          Copyright(C) 2014, Tark
//
//     [JWݒ
//
//---------------------------------------------------------------------------

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdio.h>
#include <math.h>
#include "MQPlugin.h"
#include "MQWidget.h"
#include "MQSetting.h"

#include <unordered_map>
#include <sstream>

#include "CMQObject.h"
#include "SetLocalAxis.h"

//---------------------------------------------------------------------------
//  MQGetPlugInID
//---------------------------------------------------------------------------
void SetLocalAxis::GetPlugInID(DWORD *Product, DWORD *ID)
{
	*Product = 0xE3F7A830;
	*ID      = 0xBD9A187A;
}

//---------------------------------------------------------------------------
//  MQGetPlugInName
//---------------------------------------------------------------------------
const char *SetLocalAxis::GetPlugInName(void)
{
	// vOC
	return "SetLocalAxis ver.1.0a Copyright(C) 2014, Tark.";
}

//---------------------------------------------------------------------------
//  MQEnumString
//---------------------------------------------------------------------------
const char *SetLocalAxis::EnumString(int index)
{
	switch(index){
	case 0: return "SetLocalAxis"; break;
	}
	return NULL;
}


MQBasePlugin *GetPluginClass()
{
	static SetLocalAxis plugin;
	return &plugin;
}

BOOL SetLocalAxis::Execute(int index, MQDocument doc_)
{
	switch(index){
	case 0:
		{
			return main(doc_);
		}
	}
	return FALSE;
}


SetLocalAxis::SetLocalAxis() : posX(-1), posY(-1), m_mode(-1), m_setto(-1), m_targetobj(-1), m_transform(false), m_global(false)
{
}

SetLocalAxis::~SetLocalAxis()
{
}

BOOL SetLocalAxis::main(MQDocument doc_)
{
	if(!callDialog(doc_)) {
		return FALSE;
	}
	cmq::CMQDocument doc(doc_);
	cmq::CMQObject cur = doc.getCurrentObject();
	if(!cur.isLive()) {
		return FALSE;
	}
	switch(m_mode) {
	case selV: return trans_vertex(doc, cur, true);
	case selO: return trans_vertex(doc, cur, false);
//	case selcopy: return trans_copy(doc, cur);
	case rotateL: return rotate_line(doc, cur);
	case rotateN: return rotate_normal(doc, cur);
	case rotateV: return rotate_vertex(doc, cur);
	case rotateC: return rotate_camera(doc, cur);
//	case rotatecopy: return rotate_copy(doc, cur);
	case axisX:
	case axisY:
	case axisZ: return rotateaxis(doc, cur);
	case mirrorX:
	case mirrorY:
	case mirrorZ: return mirroraxis(doc, cur);
	case copy: return copylocal(doc, cur);
	case storeClip: return storeclicpboard(doc, cur);
	case restoreClip: return restoreclicpboard(doc, cur);
	case locclear: return clearlocal(doc, cur);
	}
	return TRUE;
}

// [JWI_̒ɂ
BOOL SetLocalAxis::trans_vertex(cmq::CMQDocument &doc, cmq::CMQObject &cur, bool selected)
{
	cmq::CMQObject obj = (m_targetobj == -1) ? cur : doc.getObjectFromUniqueID(m_targetobj);
	MQPoint pmax(-FLT_MAX, -FLT_MAX, -FLT_MAX);
	MQPoint pmin(FLT_MAX, FLT_MAX, FLT_MAX);
	cmq::CMQContainer<cmq::CMQVertex, cmq::CMQObject> vcontainer(&obj);
	for(auto vit = vcontainer.begin(); vit != vcontainer.end(); ++vit) {
		if(!selected || vit->isSelect()) {
			MQPoint p = obj.getVertex(vit.index());
			pmin.x = min(p.x, pmin.x);
			pmin.y = min(p.y, pmin.y);
			pmin.z = min(p.z, pmin.z);
			pmax.x = max(p.x, pmax.x);
			pmax.y = max(p.y, pmax.y);
			pmax.z = max(p.z, pmax.z);
		}
	}
	MQPoint p = (pmin + pmax) / 2.0;
	if(m_transform) {
		translateobj(cur, p);
	}
	cur.setTranslation(p);
	return TRUE;
}

// [JWI2_ԃxNgŃZbg
BOOL SetLocalAxis::rotate_line(cmq::CMQDocument &doc, cmq::CMQObject &cur)
{
	cmq::CMQObject obj = (m_targetobj == -1) ? cur : doc.getObjectFromUniqueID(m_targetobj);
	bool isfirst = true;
	MQPoint p1, p2;
	cmq::CMQContainer<cmq::CMQVertex, cmq::CMQObject> vcontainer(&obj);
	for(auto vit = vcontainer.begin(); vit != vcontainer.end(); ++vit) {
		if(vit->isSelect()) {
			if(isfirst) {
				isfirst = false;
				p1 = obj.getVertex(vit.index());
			}
			else {
				p2 = obj.getVertex(vit.index());
				MQPoint vec = p1 - p2;
				setrotate(cur, vec);
				return TRUE;
			}
		}
	}
	cmq::CMQContainer<cmq::CMQFace, cmq::CMQObject> fcontainer(&obj);
	for(auto fit = fcontainer.begin(); fit != fcontainer.end(); ++fit) {
		int lcount = fit->getFacePointCount();
		for(int i = 0; i < lcount; i++) {
			if(doc.isSelectLine(obj, fit.index(), i)) {
				MQPoint vec = obj.getLineVector(fit.index(), i);
				setrotate(cur, vec);
				return TRUE;
			}
		}
	}
	return FALSE;
}

// [JWIʖ@ŃZbg
BOOL SetLocalAxis::rotate_normal(cmq::CMQDocument &doc, cmq::CMQObject &cur)
{
	cmq::CMQObject obj = (m_targetobj == -1) ? cur : doc.getObjectFromUniqueID(m_targetobj);
	cmq::CMQContainer<cmq::CMQFace, cmq::CMQObject> fcontainer(&obj);
	for(auto fit = fcontainer.begin(); fit != fcontainer.end(); ++fit) {
		if(fit->isSelect()) {
			MQPoint normal = obj.getNormalVector(fit.index());
			setrotate(cur, normal);
			return TRUE;
		}
	}
	int pcount = 0;
	MQPoint p[3];
	cmq::CMQContainer<cmq::CMQVertex, cmq::CMQObject> vcontainer(&obj);
	for(auto vit = vcontainer.begin(); vit != vcontainer.end(); ++vit) {
		if(vit->isSelect()) {
			if(pcount < 3) {
				p[pcount] = obj.getVertex(vit.index());
				if(++pcount == 3) {
					MQPoint vec = cmq::CMQPoint::outercross(p[1] - p[0], p[2] - p[0]);
					setrotate(cur, vec);
					return TRUE;
				}
			}
		}
	}
	return FALSE;
}

// [JW(JgIuWFNg)[J_I𒸓_ւ̃xNgŃZbg
BOOL SetLocalAxis::rotate_vertex(cmq::CMQDocument &doc, cmq::CMQObject &cur)
{
	cmq::CMQObject obj = (m_targetobj == -1) ? cur : doc.getObjectFromUniqueID(m_targetobj);
	cmq::CMQContainer<cmq::CMQVertex, cmq::CMQObject> vcontainer(&obj);
	for(auto vit = vcontainer.begin(); vit != vcontainer.end(); ++vit) {
		if(vit->isSelect()) {
			MQPoint ptarget = obj.getVertex(vit.index()); // ^[Qbg̒_
			MQPoint p0 = cur.getTranslation(); // JgIuWFNg̃[J_
			MQPoint vec = ptarget - p0;
			setrotate(cur, vec);
			return TRUE;
		}
	}
	return FALSE;
}

// [JWJ_ŃZbg
BOOL SetLocalAxis::rotate_camera(cmq::CMQDocument &doc, cmq::CMQObject &cur)
{
	MQScene scene = doc.getScene(0);
	MQAngle angle = scene->GetCameraAngle();
	MQMatrix r;
	MQPoint scale(1.0, 1.0, 1.0);
	MQPoint trans(0.0, 0.0, 0.0);
	r.SetTransform(&scale, &angle, &trans);
	MQPoint vec = r.GetRow3(2);
	vec.z = -vec.z;
	setrotate(cur, vec);
	return TRUE;
}

// [Jւ
BOOL SetLocalAxis::rotateaxis(cmq::CMQDocument &doc, cmq::CMQObject &cur)
{
	MQMatrix r = cur.getLocalMatrix();
	MQPoint ex = r.GetRow3(0);
	MQPoint ey = r.GetRow3(1);
	MQPoint ez = r.GetRow3(2);
	switch(m_mode) {
	case axisX: 
		r.SetRow3(1, ez);
		r.SetRow3(2, -ey);
		break;
	case axisY:
		r.SetRow3(0, -ez);
		r.SetRow3(2, ex);
		break;
	case axisZ: 
		r.SetRow3(0, ey);
		r.SetRow3(1, -ex);
		break;
	}
	if(m_transform) {
		rotateobj(cur, r);
	}
	cur.setLocalMatrix(r);
	return TRUE;
}

// ~[
BOOL SetLocalAxis::mirroraxis(cmq::CMQDocument &doc, cmq::CMQObject &cur)
{
	MQMatrix r = cur.getLocalMatrix();
	switch(m_mode) {
	case mirrorX: 
		r.SetColumn3(0, -r.GetColumn3(0));
		if(m_global) {
			r._41 = -r._41;
		}
		break;
	case mirrorY:
		r.SetColumn3(1, -r.GetColumn3(1));
		if(m_global) {
			r._42 = -r._42;
		}
		break;
	case mirrorZ: 
		r.SetColumn3(2, -r.GetColumn3(2));
		if(m_global) {
			r._43 = -r._43;
		}
		break;
	}
	if(m_transform) {
		rotateobj(cur, r, true);
	}
	cur.setLocalMatrix(r);
	return TRUE;
}

// IIuWFNgփ[JWݒRs[
BOOL SetLocalAxis::copylocal(cmq::CMQDocument &doc, cmq::CMQObject &cur)
{
	cmq::CMQObject obj = (m_targetobj == -1) ? cur : doc.getObjectFromUniqueID(m_targetobj);
	MQPoint t = obj.getTranslation();
	MQMatrix r = obj.getLocalMatrix();
	for(int ocount = 0; ocount < doc.getObjectCount(); ocount++) {
		cmq::CMQObject &target = doc.getObject(ocount);
		if(!target.isLive() || target == obj || !target.getSelected()) {
			continue;
		}
		MQMatrix rtarget = target.getLocalMatrix();
		MQPoint scale = m_copyS ? r.GetScaling() : rtarget.GetScaling();
		MQAngle angle = m_copyR ? r.GetRotation() : rtarget.GetRotation();
		MQPoint translate = m_copyT ? r.GetTranslation() : rtarget.GetTranslation();
		r.SetTransform(&scale, &angle, &translate);
		if(m_transform) {
			rotateobj(target, r, true);
		}
		target.setLocalMatrix(r);
	}
	return TRUE;
}

// [JWNA
BOOL SetLocalAxis::clearlocal(cmq::CMQDocument &doc, cmq::CMQObject &cur)
{
	cmq::CMQObject obj = (m_targetobj == -1) ? cur : doc.getObjectFromUniqueID(m_targetobj);
	MQMatrix r;
	for(int i = 0; i < 16; i++) {
		r.t[i]= 0.0;
	}
	r._11 = r._22 = r._33 = r._44 = 1.0f;
	if(m_transform) {
		rotateobj(obj, r, true);
	}
	obj.setLocalMatrix(r);
	return TRUE;
}

// Nbv{[hփ[JϊsZbg
BOOL SetLocalAxis::storeclicpboard(cmq::CMQDocument &doc, cmq::CMQObject &cur)
{
	cmq::CMQObject obj = (m_targetobj == -1) ? cur : doc.getObjectFromUniqueID(m_targetobj);
	MQMatrix r = obj.getLocalMatrix();
	std::ostringstream ss;
	for(int i = 0; i < 16; i++) {
		if(i > 0) {
			ss << ',';
		}
		ss << r.t[i];
	}
	setClipboardText(ss.str());
	return FALSE;
}

// Nbv{[h烍[Jϊs擾ăZbg
BOOL SetLocalAxis::restoreclicpboard(cmq::CMQDocument &doc, cmq::CMQObject &cur)
{
	cmq::CMQObject obj = (m_targetobj == -1) ? cur : doc.getObjectFromUniqueID(m_targetobj);
	MQMatrix r;
	std::istringstream ss(getClipboardText());
	char c;
	for(int i = 0; i < 16; i++) {
		ss >> r.t[i];
		if(ss.fail()) {
			return FALSE;
		}
		ss >> c;
	}
	MQMatrix rtarget = obj.getLocalMatrix();
	MQPoint scale = m_copyS ? r.GetScaling() : rtarget.GetScaling();
	MQAngle angle = m_copyR ? r.GetRotation() : rtarget.GetRotation();
	MQPoint translate = m_copyT ? r.GetTranslation() : rtarget.GetTranslation();
	r.SetTransform(&scale, &angle, &translate);
	if(m_transform) {
		rotateobj(obj, r, true);
	}
	obj.setLocalMatrix(r);
	return TRUE;
}

// Nbv{[hփeLXgZbg
void SetLocalAxis::setClipboardText(const std::string &s)
{
	HGLOBAL hText = GlobalAlloc(GMEM_DDESHARE | GMEM_MOVEABLE, s.size() + 1);
	if(!hText) {
		return;
	}
	char *p = (char*)GlobalLock(hText);
	if(!p) {
		return;
	}
	strcpy_s(p, s.size() + 1, s.c_str());
	GlobalUnlock(hText);
	if(OpenClipboard(NULL)) {
		EmptyClipboard();
		SetClipboardData(CF_TEXT, hText);
		CloseClipboard();
	}
}

// Nbv{[heLXg擾
std::string SetLocalAxis::getClipboardText(void)
{
	std::string s;
	if(OpenClipboard(nullptr)) {
		HANDLE hText = GetClipboardData(CF_TEXT);
		if(hText) {
			char *p = (char*)GlobalLock(hText);
			s = p;
			GlobalUnlock(hText);
		}
		CloseClipboard();
	}
	return s;
}

// IuWFNg𕽍sړ
void SetLocalAxis::translateobj(cmq::CMQObject &obj, MQPoint trans)
{
	trans -= obj.getTranslation(); // ݈ʒu̍
	cmq::CMQContainer<cmq::CMQVertex, cmq::CMQObject> vcontainer(&obj);
	for(auto vit = vcontainer.begin(); vit != vcontainer.end(); ++vit) {
		MQPoint p = obj.getVertex(vit.index());
		obj.setVertex(vit.index(), p + trans);
	}
}

// IuWFNg]
void SetLocalAxis::rotateobj(cmq::CMQObject &obj, MQMatrix r, bool move)
{
	MQMatrix ri = obj.getLocalInverseMatrix(); // ݂̃[Jϊts
	MQMatrix r0 = obj.getLocalMatrix();
	MQPoint scale0 = r0.GetScaling();
//	MQAngle angle0 = r0.GetRotation();
	MQPoint translate0 = r0.GetTranslation();
	MQPoint scale = r.GetScaling();
	MQAngle angle = r.GetRotation();
	MQPoint translate = r.GetTranslation();
	r.SetTransform(move ? &scale : &scale0,
		&angle,
		move ? &translate : &translate0); // ]ĈړȂ悤ɁA̍WRs[

	r = ri * r;
	cmq::CMQContainer<cmq::CMQVertex, cmq::CMQObject> vcontainer(&obj);
	for(auto vit = vcontainer.begin(); vit != vcontainer.end(); ++vit) {
		MQPoint p = obj.getVertex(vit.index());
		p = p * r;
		obj.setVertex(vit.index(), p);
	}
}

// [JWw肷
// YwcYvecŃZbg
// XwcY͌Œ̂܂܁AXvecYɐɃZbg
void SetLocalAxis::setrotate(cmq::CMQObject &obj, MQPoint &vec)
{
	MQMatrix r = obj.getLocalMatrix();
	if(m_setto == 0) {
		// Yݒ
		MQPoint ex = r.GetRow3(0);
		MQPoint ez = cmq::CMQPoint::outercross(ex, vec);
		ex = cmq::CMQPoint::outercross(vec, ez);
		ex.normalize();
		vec.normalize();
		ez.normalize();
		r.SetRow3(0, ex);
		r.SetRow3(1, vec);
		r.SetRow3(2, ez);
	}
	else {
		// YŒ肵Xݒ
		MQPoint ey = r.GetRow3(1);
		MQPoint ez = cmq::CMQPoint::outercross(vec, ey);
		vec = cmq::CMQPoint::outercross(ey, ez);
		vec.normalize();
		ey.normalize();
		ez.normalize();
		r.SetRow3(0, vec);
		r.SetRow3(1, ey);
		r.SetRow3(2, ez);
	}
	if(m_transform) {
		rotateobj(obj, r);
	}
	obj.setLocalMatrix(r);
}

void SetLocalAxis::loadSetting()
{
	MQSetting *setting = OpenSetting();
	setting->Load("PosX", posX, -1);
	setting->Load("PosY", posY, -1);
	setting->Load("transform", m_transform, false);
	setting->Load("global", m_global, false);
	setting->Load("copyR", m_copyR, true);
	setting->Load("copyS", m_copyS, false);
	setting->Load("copyT", m_copyT, false);
	setting->Load("target", m_targetobj, -1);
	CloseSetting(setting);
}

void SetLocalAxis::saveSetting()
{
	MQSetting *setting = OpenSetting();
	setting->Save("PosX", posX);
	setting->Save("PosY", posY);
	setting->Save("transform", m_transform);
	setting->Save("global", m_global);
	setting->Save("copyR", m_copyR);
	setting->Save("copyS", m_copyS);
	setting->Save("copyT", m_copyT);
	setting->Save("target", m_targetobj);
	CloseSetting(setting);
}

BOOL SetLocalAxis::callDialog(MQDocument doc)
{
	MQWindow mainwin = MQWindow::GetMainWindow();
	SetLocalAxisDialog dlg(mainwin, this);
	loadSetting();
	if(posX >= 0 && posY >= 0) {
		dlg.SetPosX(posX);
		dlg.SetPosY(posY);
	}
	dlg.listup(doc, m_targetobj);
	dlg.check(nullptr, doc);
	dlg.m_check_transform->SetChecked(m_transform);
	dlg.m_check_global->SetChecked(m_global);
	dlg.m_check_copyR->SetChecked(m_copyR);
	dlg.m_check_copyS->SetChecked(m_copyS);
	dlg.m_check_copyT->SetChecked(m_copyT);
	MQDialog::DIALOG_RESULT result = dlg.Execute();
	m_mode = dlg.mode;
	if(dlg.m_radio_setY->GetChecked()) {
		m_setto = 0;
	}
	else {
		m_setto = 1;
	}
	m_targetobj = (UINT)dlg.m_combo_targetobj->GetItemTag(dlg.m_combo_targetobj->GetCurrentIndex());
	m_transform = dlg.m_check_transform->GetChecked();
	m_global = dlg.m_check_global->GetChecked();
	m_copyR = dlg.m_check_copyR->GetChecked();
	m_copyS = dlg.m_check_copyS->GetChecked();
	m_copyT = dlg.m_check_copyT->GetChecked();
	posX = dlg.GetPosX();
	posY = dlg.GetPosY();
	saveSetting();
	return result == MQDialog::DIALOG_OK ? TRUE : FALSE;
}

//------------------------------------------------------------------------------

SetLocalAxisDialog::SetLocalAxisDialog(MQWindowBase& parent, SetLocalAxis *plugin)
	: MQDialog(parent), m_pPlugin(plugin), mode(-1)
{
	setlocale(LC_CTYPE, "");
	SetTitle( L"SetLocalAxis");
	SetInSpace(0.0);
	SetOutSpace(0.0);

	MQFrame *mainFrame = CreateVerticalFrame(this);
	{
		MQFrame *frame3 = CreateHorizontalFrame(mainFrame);
		MQLabel *labe = CreateLabel(frame3, L"Reference object");
		m_combo_targetobj = CreateComboBox(frame3);
		m_combo_targetobj->SetHorzLayout(LAYOUT_FILL);
		m_combo_targetobj->AddChangedEvent(this, &SetLocalAxisDialog::check);
	}
	{
		MQGroupBox *group1 = CreateGroupBox(mainFrame, L"set origin");

		m_button_transvert = CreateButton(group1, L"Center of (&V)ertexes");
		m_button_transvert->AddClickEvent(this, &SetLocalAxisDialog::click);
		m_button_transvert->SetModalResult(DIALOG_OK);

		m_button_transobject = CreateButton(group1, L"Center of the O(&b)ject");
		m_button_transobject->AddClickEvent(this, &SetLocalAxisDialog::click);
		m_button_transobject->SetModalResult(DIALOG_OK);
	}
	{
		MQGroupBox *group2 = CreateGroupBox(mainFrame, L"set axis");
		m_button_rotateline = CreateButton(group2, L"(&L)ine");
		m_button_rotateline->AddClickEvent(this, &SetLocalAxisDialog::click);
		m_button_rotateline->SetModalResult(DIALOG_OK);

		m_button_rotatenormal = CreateButton(group2, L"(&N)ormal of a Face");
		m_button_rotatenormal->AddClickEvent(this, &SetLocalAxisDialog::click);
		m_button_rotatenormal->SetModalResult(DIALOG_OK);

		m_button_rotatevert = CreateButton(group2, L"(&O)rigin to Vertex");
		m_button_rotatevert->AddClickEvent(this, &SetLocalAxisDialog::click);
		m_button_rotatevert->SetModalResult(DIALOG_OK);

		m_button_rotatecamera = CreateButton(group2, L"(&C)amera");
		m_button_rotatecamera->AddClickEvent(this, &SetLocalAxisDialog::click);
		m_button_rotatecamera->SetModalResult(DIALOG_OK);

		MQFrame *frame1 = CreateHorizontalFrame(group2);
		m_radio_setY = CreateRadioButton(frame1, L"set to Y(&A)");
		m_radio_setY->AddChangedEvent(this, &SetLocalAxisDialog::radio);
		m_radio_setY->SetChecked(true);
		m_radio_setY->SetFillBeforeRate(0.2);
		m_radio_setX = CreateRadioButton(frame1, L"set to X");
		m_radio_setX->AddChangedEvent(this, &SetLocalAxisDialog::radio);
		m_radio_setX->SetFillAfterRate(0.2);
	}
	{
		MQGroupBox *group2 = CreateGroupBox(mainFrame, L"copy to selected objects");
		group2->SetAlignment(MQGroupBox::ALIGN_HORIZONTAL);

		m_button_copy = CreateButton(group2, L"Copy");
		m_button_copy->AddClickEvent(this, &SetLocalAxisDialog::click);
		m_button_copy->SetModalResult(DIALOG_OK);

		m_check_copyR = CreateCheckBox(group2, L"(&R)otate");
		m_check_copyS = CreateCheckBox(group2, L"(&S)cale");
		m_check_copyT = CreateCheckBox(group2, L"(&T)rans");
	}
	{
		MQGroupBox *group3 = CreateGroupBox(mainFrame, L"axis");

		MQFrame *frame2 = CreateHorizontalFrame(group3);
		MQLabel *label1 = CreateLabel(frame2, L"Rotate");

		m_button_rotateaxisX = CreateButton(frame2, L"&X");
		m_button_rotateaxisX->AddClickEvent(this, &SetLocalAxisDialog::click);
		m_button_rotateaxisX->SetModalResult(DIALOG_OK);

		m_button_rotateaxisY = CreateButton(frame2, L"&Y");
		m_button_rotateaxisY->AddClickEvent(this, &SetLocalAxisDialog::click);
		m_button_rotateaxisY->SetModalResult(DIALOG_OK);

		m_button_rotateaxisZ = CreateButton(frame2, L"&Z");
		m_button_rotateaxisZ->AddClickEvent(this, &SetLocalAxisDialog::click);
		m_button_rotateaxisZ->SetModalResult(DIALOG_OK);

		MQFrame *frame3 = CreateHorizontalFrame(group3);
		MQLabel *label2 = CreateLabel(frame3, L"Mirror");

		m_button_mirroraxisX = CreateButton(frame3, L"X");
		m_button_mirroraxisX->AddClickEvent(this, &SetLocalAxisDialog::click);
		m_button_mirroraxisX->SetModalResult(DIALOG_OK);

		m_button_mirroraxisY = CreateButton(frame3, L"Y");
		m_button_mirroraxisY->AddClickEvent(this, &SetLocalAxisDialog::click);
		m_button_mirroraxisY->SetModalResult(DIALOG_OK);

		m_button_mirroraxisZ = CreateButton(frame3, L"Z");
		m_button_mirroraxisZ->AddClickEvent(this, &SetLocalAxisDialog::click);
		m_button_mirroraxisZ->SetModalResult(DIALOG_OK);

		m_check_global = CreateCheckBox(frame3, L"(&g)lobal");
	}
	MQFrame *frame4 = CreateHorizontalFrame(mainFrame);
	{
		MQGroupBox *group4 = CreateGroupBox(frame4, L"clipboard");
		MQFrame *frame3 = CreateHorizontalFrame(group4);
		m_button_store = CreateButton(frame3, L"S");
		m_button_store->AddClickEvent(this, &SetLocalAxisDialog::click);
		m_button_store->SetModalResult(DIALOG_OK);

		m_button_restore = CreateButton(frame3, L"R");
		m_button_restore->AddClickEvent(this, &SetLocalAxisDialog::click);
		m_button_restore->SetModalResult(DIALOG_OK);
	}
	m_button_clear = CreateButton(frame4, L"Clear");
	m_button_clear->AddClickEvent(this, &SetLocalAxisDialog::click);
	m_button_clear->SetModalResult(DIALOG_OK);

	m_check_transform = CreateCheckBox(frame4, L"Trans(&f)orm");

	MQButton *button_cancel = CreateButton(frame4, L"Cancel");
	button_cancel->SetCancel(true);
	button_cancel->SetModalResult(DIALOG_CANCEL);

	this->AddKeyDownEvent(this, &SetLocalAxisDialog::keyevent);
}

void SetLocalAxisDialog::listup(MQDocument doc_, UINT targetuid)
{
	cmq::CMQDocument doc(doc_);
	cmq::CMQObject cur = doc.getCurrentObject();
	m_combo_targetobj->AddItem(L"-", -1);
	int current = 0;
	for(int ocount = 0; ocount < doc.getObjectCount(); ocount++) {
		cmq::CMQObject &target = doc.getObject(ocount);
		if(!target.isLive() || target == cur) {
			continue;
		}
		m_combo_targetobj->AddItem(cmq::towstring(target.getName()), target.getUniqueID());
		if(targetuid == target.getUniqueID()) {
			current = m_combo_targetobj->GetItemCount() - 1;
		}
	}
	m_combo_targetobj->SetCurrentIndex(current);
}

BOOL SetLocalAxisDialog::click(MQWidgetBase *sender, MQDocument doc)
{
	if(sender == m_button_transvert) {mode = SetLocalAxis::selV;}
	else if(sender == m_button_transobject) {mode = SetLocalAxis::selO;}
//	else if(sender == m_button_transcopy) {mode = SetLocalAxis::selcopy;}
	else if(sender == m_button_rotateline) {mode = SetLocalAxis::rotateL;}
	else if(sender == m_button_rotatenormal) {mode = SetLocalAxis::rotateN;}
	else if(sender == m_button_rotatevert) {mode = SetLocalAxis::rotateV;}
	else if(sender == m_button_rotatecamera) {mode = SetLocalAxis::rotateC;}
//	else if(sender == m_button_rotatecopy ) {mode = SetLocalAxis::rotatecopy;}
	else if(sender == m_button_rotateaxisX) {mode = SetLocalAxis::axisX;}
	else if(sender == m_button_rotateaxisY) {mode = SetLocalAxis::axisY;}
	else if(sender == m_button_rotateaxisZ) {mode = SetLocalAxis::axisZ;}
	else if(sender == m_button_mirroraxisX) {mode = SetLocalAxis::mirrorX;}
	else if(sender == m_button_mirroraxisY) {mode = SetLocalAxis::mirrorY;}
	else if(sender == m_button_mirroraxisZ) {mode = SetLocalAxis::mirrorZ;}
	else if(sender == m_button_copy) {mode = SetLocalAxis::copy;}
	else if(sender == m_button_store) {mode = SetLocalAxis::storeClip;}
	else if(sender == m_button_restore) {mode = SetLocalAxis::restoreClip;}
	else if(sender == m_button_clear) {mode = SetLocalAxis::locclear;}

	return FALSE;
}

BOOL SetLocalAxisDialog::radio(MQWidgetBase *sender, MQDocument doc)
{
	m_radio_setX->SetChecked(sender == m_radio_setX);
	m_radio_setY->SetChecked(sender == m_radio_setY);
	return FALSE;
}

BOOL SetLocalAxisDialog::check(MQWidgetBase *sender, MQDocument doc_)
{
	UINT uid = (UINT)m_combo_targetobj->GetItemTag(m_combo_targetobj->GetCurrentIndex());
	cmq::CMQDocument doc(doc_);
	cmq::CMQObject obj = (uid == -1) ? doc.getCurrentObject() : doc.getObjectFromUniqueID(uid);
	int vcount = 0;
	int fcount = 0;
	int lcount = 0;
	cmq::CMQContainer<cmq::CMQVertex, cmq::CMQObject> vcontainer(&obj);
	for(auto vit = vcontainer.begin(); vit != vcontainer.end(); ++vit) {
		if(vit->isSelect()) {
			vcount++;
		}
	}
	cmq::CMQContainer<cmq::CMQFace, cmq::CMQObject> fcontainer(&obj);
	for(auto fit = fcontainer.begin(); fit != fcontainer.end(); ++fit) {
		if(fit->isSelect()) {
			fcount++;
		}
		int lcount = fit->getFacePointCount();
		for(int i = 0; i < lcount; i++) {
			if(doc.isSelectLine(obj, fit.index(), i)) {
				lcount++;
			}
		}
	}
	m_button_transvert->SetEnabled(vcount > 0);
	m_button_rotateline->SetEnabled(vcount == 2 || lcount == 1);
	m_button_rotatenormal->SetEnabled(vcount == 3 || fcount == 1);
	m_button_rotatevert->SetEnabled(vcount == 1);
	return FALSE;
}

BOOL SetLocalAxisDialog::keyevent(MQWidgetBase *sender, MQDocument doc, MQWidgetKeyParam& param)
{
	switch(param.Key) {
	case 0x46: m_check_transform->SetChecked(!m_check_transform->GetChecked()); break;
	case 0x41: m_radio_setX->GetChecked() ? radio(m_radio_setY, doc) : radio(m_radio_setX, doc); break;
	case 0x58: mode = SetLocalAxis::axisX; Close(DIALOG_OK); break;
	case 0x59: mode = SetLocalAxis::axisY; Close(DIALOG_OK); break;
	case 0x5A: mode = SetLocalAxis::axisZ; Close(DIALOG_OK); break;
	case 0x47: m_check_global->SetChecked(!m_check_global->GetChecked()); break;
	case 0x52: m_check_copyR->SetChecked(!m_check_copyR->GetChecked()); break;
	case 0x53: m_check_copyS->SetChecked(!m_check_copyS->GetChecked()); break;
	case 0x54: m_check_copyT->SetChecked(!m_check_copyT->GetChecked()); break;
	case 0x56: if(m_button_transvert->GetEnabled()) {mode = SetLocalAxis::selV; Close(DIALOG_OK);} break;
	case 0x42: mode = SetLocalAxis::selO; Close(DIALOG_OK); break;
	case 0x4C: if(m_button_rotateline->GetEnabled()) {mode = SetLocalAxis::rotateL; Close(DIALOG_OK);} break;
	case 0x4E: if(m_button_rotatenormal->GetEnabled()) {mode = SetLocalAxis::rotateN; Close(DIALOG_OK);} break;
	case 0x4F: if(m_button_rotatevert->GetEnabled()) {mode = SetLocalAxis::rotateV; Close(DIALOG_OK);} break;
	case 0x43: mode = SetLocalAxis::rotateC; Close(DIALOG_OK); break;
	}
	if(param.Ctrl) {
		switch(param.Key) {
		case 0x43: mode = SetLocalAxis::storeClip; Close(DIALOG_OK); break;
		case 0x56: mode = SetLocalAxis::restoreClip; Close(DIALOG_OK); break;
		}
	}
	return FALSE;
}

